#!/usr/bin/env python

import sys
import getopt
import email
import email.Parser
from email.Utils import parseaddr, formataddr
from email.Header import Header


_verbose = False
_usage = """convert mbox files that WZ Mail exports into standard mbox format.

wzmbox.py [options] [files...]
  -v --verbose    show progress information.
  -? -h --help    show this help.

mshibata@emptypage.jp
"""

_wz_charset = 'cp932'
_default_charset = 'iso-2022-jp'


def quote_header(data, charset):
    if isinstance(data, str):
        data = data.decode(charset)
    try:
        return data.encode('ascii')
    except UnicodeEncodeError:
        return str(Header(data, charset))


def quote_address_header(data, charset):
    if isinstance(data, str):
        data = data.decode(charset)
    addrs = []
    for addr in data.split(','):
        name, mailto = parseaddr(addr)
        addrs.append(formataddr((quote_header(name, charset), mailto)))
    return ', '.join(addrs)


class MediaType:
    # cf. http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.7
    
    def __init__(self, data=None):
        self.type = ''
        self.subtype = ''
        self.attrs = {}
        self.get = self.attrs.get
        if data:
            self.parse(data)
    
    
    def parse(self, data):
        if ';' not in data:
            data += ';'
        
        types, rest = data.split(';', 1)
        
        self.type, self.subtype = types.strip().lower().split('/')
        
        for pair in rest.split(';'):
            pair = pair.strip()
            if not pair:
                continue
            k, v = pair.split('=', 1)
            if v.startswith('"'):
                v = v[1:-1]
            self.attrs[k] = v
    
    
    def __quote_value(self, data):
        if ' ' in data:
            return '"%s"' % data
        else:
            return data
    
    
    def __str__(self):
        tokens = ['/'.join(self.type, self.subtype)]
        for k, v in self.attrs.values():
            tokens.append('='.join(k, self.__quote_value(v)))
        return ';'.join(tokens)


class Parser:
    
    def feed(self, fp):
        lines = []
        for line in fp:
            line = line.replace('\r\n', '\n')
            if line == '.\n':
                data = ''.join(lines)
                if data.startswith('+OK '):
                    # nanikore?
                    data = data.split('\n', 1)[1]
                self.handle_raw(data)
                lines = []
            else:
                if line.startswith('.'):
                    lines.append(line[1:])
                else:
                    lines.append(line)
    
    
    def handle_raw(self, data):
        raise NotImplementedError


class Converter(Parser):
    
    def handle_raw(self, data):
        parser = email.Parser.Parser()
        
        headers = parser.parsestr(data, headersonly=True)
        content_type = headers.get('Content-Type')
        if content_type:
            charset = MediaType(content_type).get('charset', _default_charset)
        else:
            charset = _default_charset
        if _verbose:
            print >>sys.stderr, 'converting: %s' % headers.get('Message-Id')
        if charset:
            src = data.decode(_wz_charset, 'ignore').encode(charset, 'ignore')
        else:
            src = data
        
        msg = parser.parsestr(src)
        if charset:
            if 'Subject' in msg:
                subject = msg['Subject']
                del msg['Subject']
                msg['Subject'] = quote_header(subject, charset)
            for headername in ['From', 'To', 'Cc', 'Bcc', 'Reply-To']:
                if headername not in msg:
                    continue
                addr = msg[headername]
                del msg[headername]
                msg[headername] = quote_address_header(addr, charset)
        
        print msg.as_string(True)


if __name__ == '__main__':
    try:
        opts, args = getopt.getopt(sys.argv[1:], '?hv', ['help', 'verbose'])
    except getopt.GetoptError:
        print _usage
        sys.exit(2)
    
    for k, v in opts:
        if k in ('-v', '--verbose'):
            _verbose = True
        if k in ('-?', '-h', '--help'):
            print _usage
            sys.exit()
    
    converter = Converter()
    if args:
        for filename in args:
            converter.feed(file(filename, 'rb'))
    else:
        converter.feed(sys.stdin)


